#ifndef _M_MODBUS_H_
#define _M_MODBUS_H_


/*
  Author:     Nima Askari
  WebSite:    http://www.github.com/NimaLTD
  Instagram:  http://instagram.com/github.NimaLTD
  Youtube:    https://www.youtube.com/channel/UCUhY7qY1klJm1d2kulr9ckw
  
  Version:    1.3.2
  
  Reversion History:
  
  (1.3.2)
  Speedup receiver, add write multiple holding
    
  (1.3.1)
  Remove delay in sending function when DMA is disabled
  
  (1.3.0)
  Add 16 bit order
  
  (1.2.1)
  bug fixed.
  
  (1.2.0)
  add read 32bit register order.
  
  (1.1.2)
  fix read holding register.
  
  (1.1.1)
  fix functions name.
  
  (1.1.0)
  add 16bit,32bit,float read.
  
  (1.0.1)
  add delay in receive routin.
  
  (1.0.0)
  First release.
*/ 

#include "stm32f2xx.h"
#include  "mmodbusConfig.h"
//#include  "usart.h"
#include  <stdbool.h>
#include  <string.h>
#include  <stdio.h>
#include  <stdint.h>
#include "FreeRTOS.h"
#include "task.h"
#define   mmodbus_delay(x) 	delay_ms(x) // vTaskDelay(x)



/*

MModbusCMD_ReadCoilStatusȦ״̬ڶȡһ߼Ȧ״̬
MModbusCMD_ReadDiscreteInputsɢ״̬ڶȡһźŵ״̬
MModbusCMD_ReadHoldingRegistersּĴڶȡһּĴֵ
MModbusCMD_ReadInputRegistersĴڶȡһĴֵ
MModbusCMD_WriteSingleCoilдȦڿƵ߼Ȧ״̬
MModbusCMD_WriteSingleRegisterдּĴд뵥ּĴֵ
MModbusCMD_WriteMultipleCoilsдȦڿƶ߼Ȧ״̬
MModbusCMD_WriteMultipleRegistersдּĴдּĴֵ
*/
typedef enum
{
  MModbusCMD_ReadCoilStatus = 1,
  MModbusCMD_ReadDiscreteInputs = 2,
  MModbusCMD_ReadHoldingRegisters = 3,
  MModbusCMD_ReadInputRegisters = 4,
  MModbusCMD_WriteSingleCoil = 5,
  MModbusCMD_WriteSingleRegister = 6,
  MModbusCMD_WriteMultipleCoils = 15,
  MModbusCMD_WriteMultipleRegisters = 16
  
}MModbusCMD_t;

//С
typedef enum
{
  MModBus_16bitOrder_AB=1,
  MModBus_16bitOrder_BA=2,
  
}MModBus_16bitOrder_t;

typedef enum
{
  MModBus_32bitOrder_ABCD = 0,
  MModBus_32bitOrder_DCBA,
  MModBus_32bitOrder_BADC,
  MModBus_32bitOrder_CDAB,  
  
}MModBus_32bitOrder_t;

/*
	rxIndexջĵǰλá
	rxBufջ
	rxTimeһνݵʱ
	txBusyǷæµı־λ
	timeoutʱʱ䡣
	byteOrder1616 λݵֽ򣨼λֽǰǵλֽǰ
	byteOrder3232 λݵֽ
	doneǷɵı־λ
	txDmaDoneDMA Ƿɵı־λóԱֻʹ DMA ʱЧ
*/
typedef struct
{
  uint16_t              rxIndex;  
  uint8_t               rxBuf[_MMODBUS_RXSIZE];
  uint32_t              rxTime;
  uint8_t               txBusy;
  uint32_t              timeout; 
  MModBus_16bitOrder_t  byteOrder16;
  MModBus_32bitOrder_t  byteOrder32;
	uint8_t								done;
  #if (_MMODBUS_TXDMA == 1)
  uint8_t             txDmaDone;
  #endif  
  
}MModBus_t;

extern MModBus_t mmodbus;
//##################################################################################################

void    mmodbus_callback(UART_HandleTypeDef *husart);
//void 		mmodbus_callback(void);
bool    mmodbus_sendRaw(uint8_t *data, uint16_t size, uint32_t timeout);
uint16_t mmodbus_crc16(const uint8_t *nData, uint16_t wLength); 
void    mmodbus_callback_txDMA(void);

bool    mmodbus_init(uint32_t setTimeout);
void    mmodbus_set16bitOrder(MModBus_16bitOrder_t MModBus_16bitOrder_);
void    mmodbus_set32bitOrder(MModBus_32bitOrder_t MModBus_32bitOrder_);
//  coils numbers 00001 to 09999
bool    mmodbus_readCoil(uint8_t slaveAddress, uint16_t number, uint8_t *data);
bool    mmodbus_readCoils(uint8_t slaveAddress, uint16_t startnumber, uint16_t length, uint8_t *data);
//  discrete input 10001 to 19999
bool    mmodbus_readDiscreteInput(uint8_t slaveAddress, uint16_t number, uint8_t *data);
bool    mmodbus_readDiscreteInputs(uint8_t slaveAddress, uint16_t startnumber, uint16_t length, uint8_t *data);
//  input register 30001 to 39999
bool    mmodbus_readInputRegister32f(uint8_t slaveAddress, uint16_t number, float *data);
bool    mmodbus_readInputRegisters32f(uint8_t slaveAddress, uint16_t startnumber, uint16_t length, float *data);
bool    mmodbus_readInputRegister32i(uint8_t slaveAddress, uint16_t number, uint32_t *data);
bool    mmodbus_readInputRegisters32i(uint8_t slaveAddress, uint16_t startnumber, uint16_t length, uint32_t *data);
bool    mmodbus_readInputRegister16i(uint8_t slaveAddress, uint16_t number, uint16_t *data);
bool    mmodbus_readInputRegisters16i(uint8_t slaveAddress, uint16_t startnumber, uint16_t length, uint16_t *data);
//  holding register 40001 to 49999
bool    mmodbus_readHoldingRegister32f(uint8_t slaveAddress, uint16_t number, float *data);
bool    mmodbus_readHoldingRegisters32f(uint8_t slaveAddress, uint16_t startnumber, uint16_t length, float *data);
bool    mmodbus_readHoldingRegister32i(uint8_t slaveAddress, uint16_t number, uint32_t *data);
bool    mmodbus_readHoldingRegisters32i(uint8_t slaveAddress, uint16_t startnumber, uint16_t length, uint32_t *data);
bool    mmodbus_readHoldingRegister16i(uint8_t slaveAddress, uint16_t number, uint16_t *data);
bool    mmodbus_readHoldingRegisters16i(uint8_t slaveAddress, uint16_t startnumber, uint16_t length, uint16_t *data);
// coils numbers 00001 to 09999
bool    mmodbus_writeCoil(uint8_t slaveAddress, uint16_t number, uint8_t data);
//  holding register 40001 to 49999
bool    mmodbus_writeHoldingRegister16i(uint8_t slaveAddress, uint16_t number, uint16_t data);
bool    mmodbus_writeHoldingRegisters16i(uint8_t slaveAddress, uint16_t startnumber, uint16_t length, uint16_t *data);
uint16_t mmodbus_crc16(const uint8_t *nData, uint16_t wLength);

//##################################################################################################
bool mmodbus_readHoldingRegisters8i(uint8_t slaveAddress, uint16_t startnumber, uint16_t length, uint8_t *data);
#endif
